
#define BOOST_TEST_MODULE MatrixMath
#include <boost/test/unit_test.hpp>

#include "../src/nemo/MatrixMath.h"
#include "../src/nemo/Norms.h"

#include <iostream>

using namespace nemo;
//using ::nemo::MatrixMath;
//using ::nemo::MatrixMath::pseudoInverse;
using namespace boost::unit_test;

// boost has non-virtual constructors ^^
#pragma GCC diagnostic ignored "-Weffc++"


BOOST_AUTO_TEST_SUITE(MatrixMath)


BOOST_AUTO_TEST_CASE(matrixPseudoInverse){
	// zero dim matrices
	RealMatrix mDim0;
	RealMatrix mDim0I = pseudoInverse(mDim0);
	BOOST_CHECK_EQUAL(mDim0, mDim0I);

	RealMatrix mDim01(dim(0,1));
	RealMatrix mDim10(dim(1,0));
	BOOST_CHECK_EQUAL(pseudoInverse(mDim01), mDim10);
	BOOST_CHECK_EQUAL(mDim01, pseudoInverse(mDim10));

	RealMatrix mId = RealMatrix::diagonal(dim(3,3), 1.0);
	BOOST_CHECK_EQUAL(pseudoInverse(mId), mId);

	RealMatrix mZero(dim(2,2));
	BOOST_CHECK_EQUAL(pseudoInverse(mZero), mZero);

	RealMatrix m1(dim(2,2));
	m1(0,0) = 2;
	m1(0,1) = 1;
	m1(1,0) = 1;
	m1(1,1) = 0;
	RealMatrix m1I(dim(2,2));
	m1I(0,0) = 0;
	m1I(0,1) = 1;
	m1I(1,0) = 1;
	m1I(1,1) = -2;
	const RealMatrix m1Iest = pseudoInverse(m1);
	BOOST_CHECK_SMALL((double)m1Iest(0,0)-(double)m1I(0,0), 1E-9);
	BOOST_CHECK_SMALL((double)m1Iest(0,1)-(double)m1I(0,1), 1E-9);
	BOOST_CHECK_SMALL((double)m1Iest(1,0)-(double)m1I(1,0), 1E-9);
	BOOST_CHECK_SMALL((double)m1Iest(1,1)-(double)m1I(1,1), 1E-9);

	RealMatrix m2(dim(2,1));
	m2(0,0) = 2;
	m2(1,0) = 1;
	RealMatrix m2I(dim(1,2));
	m2I(0,0) = 0.4;
	m2I(0,1) = 0.2;
	const RealMatrix m2Iest = pseudoInverse(m2);
	BOOST_CHECK_SMALL((double)m2Iest(0,0)-(double)m2I(0,0), 1E-9);
	BOOST_CHECK_SMALL((double)m2Iest(0,1)-(double)m2I(0,1), 1E-9);
}

class incr_gen : public Generator<int> {
public:
	incr_gen() : Generator<int>(), num(0) {}

protected:
	int generate(){
		return num++;
	}
private:
	int num;
};

BOOST_AUTO_TEST_CASE(matrixMatMatT){
	RealMatrix mDim0;
	RealMatrix m1_3(dim(1,3));
	m1_3.setRow(0, RealVector(1.0, 2.0, 3.0).transpose());
	RealMatrix m3_1(dim(3,1));
	m3_1.setCol(0, RealVector(1.0, 2.0, 3.0));

	BOOST_CHECK_EQUAL( matMatT(mDim0), mDim0 );
	BOOST_CHECK_EQUAL( matMatT(m1_3), m1_3 * m1_3.transpose() );
	BOOST_CHECK_EQUAL( matMatT(m3_1), m3_1 * m3_1.transpose() );

	Mapping<RealMatrix,double> norm = euclideanNorm<RealMatrix,double>();
	for(unsigned int rows = 0; rows <= 8; rows++){
		for(unsigned int cols = 0; cols <= 8; cols++){
			// test matrix with successive integral values
			RealMatrix m(dim(rows,cols), incr_gen());
			BOOST_CHECK_SMALL( norm( matMatT(m) - m*m.transpose() ), 1e-6 );
		}
	}
}

BOOST_AUTO_TEST_CASE(matrixMatTMat){
	RealMatrix mDim0;
	RealMatrix m1_3(dim(1,3));
	m1_3.setRow(0, RealVector(1.0, 2.0, 3.0).transpose());
	RealMatrix m3_1(dim(3,1));
	m3_1.setCol(0, RealVector(1.0, 2.0, 3.0));

	BOOST_CHECK_EQUAL( matTMat(mDim0), mDim0 );
	BOOST_CHECK_EQUAL( matTMat(m1_3), m1_3.transpose() * m1_3 );
	BOOST_CHECK_EQUAL( matTMat(m3_1), m3_1.transpose() * m3_1 );

	Mapping<RealMatrix,double> norm = euclideanNorm<RealMatrix,double>();
	for(unsigned int rows = 0; rows <= 8; rows++){
		for(unsigned int cols = 0; cols <= 8; cols++){
			// test matrix with successive integral values
			RealMatrix m(dim(rows,cols), incr_gen());
			BOOST_CHECK_SMALL( norm( matTMat(m) - m.transpose()*m ), 1e-6 );
		}
	}
}


BOOST_AUTO_TEST_SUITE_END()

